@@ -17,7 +17,6 @@
 //
 
 #include "config.h"
-#include "doomfeatures.h"
 #include "d_iwad.h"
 #include "i_system.h"
 #include "m_argv.h"
@@ -33,8 +32,6 @@ boolean W_ParseCommandLine(void)
     boolean modifiedgame = false;
     int p;
 
-#ifdef FEATURE_WAD_MERGE
-
     // Merged PWADs are loaded first, because they are supposed to be 
     // modified IWADs.
 
@@ -166,8 +163,6 @@ boolean W_ParseCommandLine(void)
         }
     }
 
-#endif
-
     //!
     // @arg <files>
     // @vanilla
